# Generated by Django 4.2.17 on 2025-04-08 05:19
# flake8: noqa

from django.db import migrations

import pfx.pfxcore.fields.media_field


class Migration(migrations.Migration):

    dependencies = [
        ('tests', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='book',
            name='cover',
            field=pfx.pfxcore.fields.media_field.MediaField(blank=True, max_length=255, null=True, verbose_name='Cover'),
        ),
    ]
